@@ -5,6 +5,7 @@ import sys
 import typing as t
 
 import sqlglot
+from sqlglot.helper import to_bool
 
 parser = argparse.ArgumentParser(description="Transpile SQL")
 parser.add_argument(
@@ -28,10 +29,11 @@ parser.add_argument(
     help="Dialect to write default is generic",
 )
 parser.add_argument(
-    "--no-identify",
+    "--identify",
     dest="identify",
-    action="store_false",
-    help="Don't auto identify fields",
+    type=str,
+    default="safe",
+    help="Whether to quote identifiers (safe, true, false)",
 )
 parser.add_argument(
     "--no-pretty",
@@ -81,13 +83,13 @@ if args.parse:
         )
     ]
 elif args.tokenize:
-    objs = sqlglot.Dialect.get_or_raise(args.read)().tokenize(sql)
+    objs = sqlglot.Dialect.get_or_raise(args.read).tokenize(sql)
 else:
     objs = sqlglot.transpile(
         sql,
         read=args.read,
         write=args.write,
-        identify=args.identify,
+        identify="safe" if args.identify == "safe" else to_bool(args.identify),
         pretty=args.pretty,
         error_level=error_level,
     )
