@@ -120,6 +120,7 @@ class _TypeAnnotator(type):
             exp.DataType.Type.DECFLOAT,
             exp.DataType.Type.DOUBLE,
             exp.DataType.Type.FLOAT,
+            exp.DataType.Type.BIGDECIMAL,
             exp.DataType.Type.DECIMAL,
             exp.DataType.Type.BIGINT,
             exp.DataType.Type.INT,
@@ -260,15 +261,16 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         # This takes care of non-traversable expressions
         self._annotate_expression(expression)
 
-        # Replace NULL type with UNKNOWN, since the former is not an actual type;
+        # Replace NULL type with the default type of the targeted dialect, since the former is not an actual type;
         # it is mostly used to aid type coercion, e.g. in query set operations.
         for expr in self._null_expressions.values():
-            expr.type = exp.DataType.Type.UNKNOWN
+            expr.type = self.dialect.DEFAULT_NULL_TYPE
 
         return expression
 
     def annotate_scope(self, scope: Scope) -> None:
         selects = {}
+
         for name, source in scope.sources.items():
             if not isinstance(source, Scope):
                 continue
@@ -288,48 +290,27 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
                 if not values:
                     continue
 
-                selects[name] = {
-                    alias: column.type
-                    for alias, column in zip(expression.alias_column_names, values)
-                }
+                alias_column_names = expression.alias_column_names
+
+                if (
+                    isinstance(expression, exp.Unnest)
+                    and not alias_column_names
+                    and expression.type
+                    and expression.type.is_type(exp.DataType.Type.STRUCT)
+                ):
+                    selects[name] = {
+                        col_def.name: t.cast(t.Union[exp.DataType, exp.DataType.Type], col_def.kind)
+                        for col_def in expression.type.expressions
+                        if isinstance(col_def, exp.ColumnDef) and col_def.kind
+                    }
+                else:
+                    selects[name] = {
+                        alias: column.type for alias, column in zip(alias_column_names, values)
+                    }
             elif isinstance(expression, exp.SetOperation) and len(expression.left.selects) == len(
                 expression.right.selects
             ):
-                selects[name] = col_types = self._setop_column_types.setdefault(id(expression), {})
-
-                if not col_types:
-                    # Process a chain / sub-tree of set operations
-                    for set_op in expression.walk(
-                        prune=lambda n: not isinstance(n, (exp.SetOperation, exp.Subquery))
-                    ):
-                        if not isinstance(set_op, exp.SetOperation):
-                            continue
-
-                        if set_op.args.get("by_name"):
-                            r_type_by_select = {
-                                s.alias_or_name: s.type for s in set_op.right.selects
-                            }
-                            setop_cols = {
-                                s.alias_or_name: self._maybe_coerce(
-                                    t.cast(exp.DataType, s.type),
-                                    r_type_by_select.get(s.alias_or_name)
-                                    or exp.DataType.Type.UNKNOWN,
-                                )
-                                for s in set_op.left.selects
-                            }
-                        else:
-                            setop_cols = {
-                                ls.alias_or_name: self._maybe_coerce(
-                                    t.cast(exp.DataType, ls.type), t.cast(exp.DataType, rs.type)
-                                )
-                                for ls, rs in zip(set_op.left.selects, set_op.right.selects)
-                            }
-
-                        # Coerce intermediate results with the previously registered types, if they exist
-                        for col_name, col_type in setop_cols.items():
-                            col_types[col_name] = self._maybe_coerce(
-                                col_type, col_types.get(col_name, exp.DataType.Type.NULL)
-                            )
+                selects[name] = self._get_setop_column_types(expression)
 
             else:
                 selects[name] = {s.alias_or_name: s.type for s in expression.selects}
@@ -477,6 +458,69 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
 
         return type2_value if type2_value in self.coerces_to.get(type1_value, {}) else type1_value
 
+    def _get_setop_column_types(
+        self, setop: exp.SetOperation
+    ) -> t.Dict[str, exp.DataType | exp.DataType.Type]:
+        """
+        Computes and returns the coerced column types for a SetOperation.
+
+        This handles UNION, INTERSECT, EXCEPT, etc., coercing types across
+        left and right operands for all projections/columns.
+
+        Args:
+            setop: The SetOperation expression to analyze
+
+        Returns:
+            Dictionary mapping column names to their coerced types
+        """
+        setop_id = id(setop)
+        if setop_id in self._setop_column_types:
+            return self._setop_column_types[setop_id]
+
+        col_types: t.Dict[str, exp.DataType | exp.DataType.Type] = {}
+
+        # Validate that left and right have same number of projections
+        if not (
+            isinstance(setop, exp.SetOperation)
+            and setop.left.selects
+            and setop.right.selects
+            and len(setop.left.selects) == len(setop.right.selects)
+        ):
+            return col_types
+
+        # Process a chain / sub-tree of set operations
+        for set_op in setop.walk(
+            prune=lambda n: not isinstance(n, (exp.SetOperation, exp.Subquery))
+        ):
+            if not isinstance(set_op, exp.SetOperation):
+                continue
+
+            if set_op.args.get("by_name"):
+                r_type_by_select = {s.alias_or_name: s.type for s in set_op.right.selects}
+                setop_cols = {
+                    s.alias_or_name: self._maybe_coerce(
+                        t.cast(exp.DataType, s.type),
+                        r_type_by_select.get(s.alias_or_name) or exp.DataType.Type.UNKNOWN,
+                    )
+                    for s in set_op.left.selects
+                }
+            else:
+                setop_cols = {
+                    ls.alias_or_name: self._maybe_coerce(
+                        t.cast(exp.DataType, ls.type), t.cast(exp.DataType, rs.type)
+                    )
+                    for ls, rs in zip(set_op.left.selects, set_op.right.selects)
+                }
+
+            # Coerce intermediate results with the previously registered types, if they exist
+            for col_name, col_type in setop_cols.items():
+                col_types[col_name] = self._maybe_coerce(
+                    col_type, col_types.get(col_name, exp.DataType.Type.NULL)
+                )
+
+        self._setop_column_types[setop_id] = col_types
+        return col_types
+
     def _annotate_binary(self, expression: B) -> B:
         left, right = expression.left, expression.right
         if not left or not right:
@@ -642,31 +686,59 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         self._set_type(expression, expr_type)
         return expression
 
+    def _annotate_subquery(self, expression: exp.Subquery) -> exp.Subquery:
+        # For scalar subqueries (subqueries with a single projection), infer the type
+        # from that single projection. This allows type propagation in cases like:
+        # SELECT (SELECT 1 AS c) AS c
+        query = expression.unnest()
+
+        if isinstance(query, exp.Query):
+            selects = query.selects
+            if len(selects) == 1:
+                self._set_type(expression, selects[0].type)
+                return expression
+
+        self._set_type(expression, exp.DataType.Type.UNKNOWN)
+        return expression
+
     def _annotate_struct_value(
         self, expression: exp.Expression
     ) -> t.Optional[exp.DataType] | exp.ColumnDef:
         # Case: STRUCT(key AS value)
+        this: t.Optional[exp.Expression] = None
+        kind = expression.type
+
         if alias := expression.args.get("alias"):
-            return exp.ColumnDef(this=alias.copy(), kind=expression.type)
+            this = alias.copy()
+        elif expression.expression:
+            # Case: STRUCT(key = value) or STRUCT(key := value)
+            this = expression.this.copy()
+            kind = expression.expression.type
+        elif isinstance(expression, exp.Column):
+            # Case: STRUCT(c)
+            this = expression.this.copy()
 
-        # Case: STRUCT(key = value) or STRUCT(key := value)
-        if expression.expression:
-            return exp.ColumnDef(this=expression.this.copy(), kind=expression.expression.type)
+        if kind and kind.is_type(exp.DataType.Type.UNKNOWN):
+            return None
 
-        # Case: STRUCT(c)
-        if isinstance(expression, exp.Column):
-            return exp.ColumnDef(this=expression.this.copy(), kind=expression.type)
+        if this:
+            return exp.ColumnDef(this=this, kind=kind)
 
-        return expression.type
+        return kind
 
     def _annotate_struct(self, expression: exp.Struct) -> exp.Struct:
+        expressions = []
+        for expr in expression.expressions:
+            struct_field_type = self._annotate_struct_value(expr)
+            if struct_field_type is None:
+                self._set_type(expression, None)
+                return expression
+
+            expressions.append(struct_field_type)
+
         self._set_type(
             expression,
-            exp.DataType(
-                this=exp.DataType.Type.STRUCT,
-                expressions=[self._annotate_struct_value(expr) for expr in expression.expressions],
-                nested=True,
-            ),
+            exp.DataType(this=exp.DataType.Type.STRUCT, expressions=expressions, nested=True),
         )
         return expression
 
