@@ -1,6 +1,6 @@
-use std::collections::{HashMap, HashSet};
-
 use pyo3::prelude::*;
+use rustc_hash::FxHashMap as HashMap;
+use rustc_hash::FxHashSet as HashSet;
 
 pub type TokenType = u16;
 
@@ -104,6 +104,7 @@ pub struct TokenizerSettings {
     pub string_escapes_allowed_in_raw_strings: bool,
     pub nested_comments: bool,
     pub hint_start: String,
+    pub escape_follow_chars: HashSet<char>,
 }
 
 #[pymethods]
@@ -130,6 +131,7 @@ impl TokenizerSettings {
         string_escapes_allowed_in_raw_strings: bool,
         nested_comments: bool,
         hint_start: String,
+        escape_follow_chars: HashSet<String>,
     ) -> Self {
         let to_char = |v: &String| {
             if v.len() == 1 {
@@ -158,6 +160,7 @@ impl TokenizerSettings {
             identifier_escapes.iter().map(&to_char).collect();
 
         let string_escapes_native: HashSet<char> = string_escapes.iter().map(&to_char).collect();
+        let escape_follow_chars_native: HashSet<char> = escape_follow_chars.iter().map(&to_char).collect();
 
         let var_single_tokens_native: HashSet<char> =
             var_single_tokens.iter().map(&to_char).collect();
@@ -183,6 +186,7 @@ impl TokenizerSettings {
             string_escapes_allowed_in_raw_strings,
             nested_comments,
             hint_start,
+            escape_follow_chars: escape_follow_chars_native,
         };
 
         #[cfg(feature = "profiling")]
