@@ -63,6 +63,10 @@ INT;
 STRUCT(1 AS col);
 STRUCT<col INT>;
 
+# Note: ensure the struct is annotated as UNKNOWN when any of its arguments are UNKNOWN
+STRUCT(1, f2);
+UNKNOWN;
+
 STRUCT(1 AS col, 2.5 AS row);
 STRUCT<col INT, row DOUBLE>;
 
@@ -106,3 +110,38 @@ DECIMAL(18,2);
 
 CASE WHEN x THEN NULL ELSE CAST(y AS DECIMAL(18, 2)) END;
 DECIMAL(18,2);
+
+# dialect: bigquery
+CASE WHEN TRUE THEN '2010-01-01' ELSE DATE '2020-02-02' END;
+DATE;
+
+# dialect: bigquery
+CASE WHEN TRUE THEN '2010-01-01' WHEN FALSE THEN DATE '2020-02-02' ELSE '1990-01-01' END;
+DATE;
+
+# dialect: bigquery
+CASE WHEN TRUE THEN DATETIME '2020-02-02 00:00:00' ELSE '2010-01-01' END;
+DATETIME;
+
+# dialect: bigquery
+CASE WHEN TRUE THEN TIMESTAMP '2020-02-02 00:00:00' ELSE '2010-01-01' END;
+TIMESTAMP;
+
+# dialect: bigquery
+NULL;
+INT64;
+
+# dialect: bigquery
+ARRAY(SELECT 'foo' UNION ALL SELECT 'bar');
+ARRAY<STRING>;
+
+# dialect: bigquery
+ARRAY(SELECT 1 UNION ALL SELECT 2 UNION ALL SELECT 3);
+ARRAY<INT64>;
+
+# dialect: bigquery
+ARRAY(SELECT 1 UNION ALL SELECT 2.5);
+ARRAY<FLOAT64>;
+
+1 + (SELECT 2.5 AS c);
+DOUBLE;
